#-----------------------------------------------------------------------------
# Name:        WeatherTool.py
# Purpose:
#
# Author:      Song Huang <Song@ossacc.org>
#
# Created:     2004/01/29
# Copyright:   (c) 2004 OSSACC.org
# Licence:     GPL
#-----------------------------------------------------------------------------

from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import aq_base, aq_inner, aq_parent, Implicit
from App.Common import package_home
from Globals import DTMLFile, InitializeClass, MessageDialog
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem, Item_w__name__
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import UniqueObject, getToolByName
from help import IZWeatherApplet
from ZWeatherApplet import ZWeatherApplet
from cStringIO import StringIO
from exceptions import RuntimeError
from pymetar import MetarReport
from zLOG import LOG
from DefaultStations import DefaultStations
import string
import time

weather_globals = globals()

class AlreadyInstalled(Exception):
    """ Would be nice to say what Product was trying to be installed """
    pass

def addWeatherTool(self, REQUEST=None):
    ''' '''
    out = StringIO()
    skinstool = getToolByName(self, 'portal_skins')
    
    # Setup the skins
    if 'weather' not in skinstool.objectIds():
        addDirectoryViews(skinstool, 'skins', weather_globals)
        out.write("Added weather skin directories to portal_skins\n")
    
    # Add Path to Skins Properties
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        for dir in ('weather',):
            if not dir in path:
                try:
                    idx = path.index('custom')
                except ValueError:
                    idx = 999
                path.insert(idx+1, dir)

        path = string.join(path, ', ')
        # addSkinSelection will replace existing skins as well.
        skinstool.addSkinSelection(skin, path)
        out.write("Added 'weather' to %s skin\n" % skin)
    
    # Add portlet
    macros = 'here/portlet_weather/macros/portlet'
    slots  = self.getProperty('right_slots')
    if self.hasProperty('right_slots') and macros not in slots:
        rs = (macros,) + slots
        self.manage_changeProperties({'right_slots':rs})

    # create weather tool
    qt=WeatherTool()
    self._setObject('portal_weather',qt,set_owner=0)
    if REQUEST:
        return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

class WeatherTool( UniqueObject, SimpleItem, PropertyManager, Item_w__name__ ):
    """WeatherTool implementation"""

    __implements__ = IZWeatherApplet.IZWeatherApplet

    _properties = (
                      {'id':'title', 'type': 'string', 'mode':'w'},
                      {'id':'station', 'type': 'lines', 'mode':'w'},
                      {'id':'minute', 'type': 'int', 'mode':'w'},
                  )

    meta_type = 'CMF Weather Tool'
    id='portal_weather'

    security = ClassSecurityInfo()

    manage_options = (
          { "label" : "Edit", "action" : "editForm" },
          { "label" : "View", "action" : "index_html"},
        ) + PropertyManager.manage_options + SimpleItem.manage_options


    security.declareProtected("View management screens", "editForm")
    editForm = DTMLFile("dtml/changeWT", globals())

    def __init__(self):
        self.id = 'portal_weather'
        self.station = DefaultStations
        self.stations = {}
        self.minute = 15
        self.editStation(self.station, self.minute)

    security.declareProtected("View", "getStation")
    def getStation(self):
        results = {}
        for s in self.station:
            results[s] = self.stations[s]['metar'].getStationName()
        return results

    security.declareProtected("View", "index_html")
    def index_html(self, station='', REQUEST={}, RESPONSE=None, **kw) :
        """Displays the ZWeatherApplet information."""
        if station:
            return self.WeatherData(station)
        else:
            return self.WeatherData(self.station[0])

    def __call__(self, station='', REQUEST={}, RESPONSE=None, **kw) :
        """Displays the ZWeatherApplet information."""
        if station and type(station) == type(''):
            return self.WeatherData(station)
        else:
            return self.WeatherData(self.station[0])

    security.declareProtected("View", "Weather")
    def WeatherData(self, station) :
        """Return weather information."""
        self.refresh(station=station)
        metar = self.stations[station]['metar']
        if metar.valid :
            idata = {}
            pix = self.getPixmapURL(station)
            if pix is not None :
                idata['pix'] = pix
            else:
                return None
            sname = metar.getStationName()
            if sname is not None :
                idata['sname'] = sname
            else:
                idata['sname'] = None
            t = metar.getTemperatureCelsius()
            if t is not None :
                idata['t'] = t
            else:
                idata['t'] = None
            h = metar.getHumidity()
            if h is not None :
                idata['h'] = h
            else:
                idata['h'] = None
            p = metar.getPressure()
            if p is not None :
                idata['p'] = p
            else:
                idata['p'] = None
            c = metar.getWindCompass()
            if c is None :
                idata['c'] = ""
            else :
                idata['c'] = "%s wind" % c
            s = metar.getWindSpeed()
            if s is not None :
                idata['s'] = s
            else:
                idata['s'] = None
            w = metar.getWeather()
            if w is not None :
                idata['w'] = w
            else:
                idata['w'] = None
            return idata

    security.declareProtected("Manage ZWeatherApplets", "CACHEDURATION")
    def CACHEDURATION(self):
        return self.minute * 60

    security.declareProtected("View", "refresh")
    def refresh(self, force=0, station=None) :
        """Refreshes the weather information from NOAA's Metar server, if needed.

           force : set this to 1 if you want to fetch a new report even if the
                   cache hasn't expired yet.
        """
        if station:
            if not self.stations.has_key(station):
                self.stations[station] = {}
                self.stations[station]['metar'] = MetarReport(station)
                self.stations[station]['lastRefresh'] = time.time()
            elif force or ((time.time() - self.stations[station]['lastRefresh']) > self.CACHEDURATION()) :
                try :
                    self.stations[station]['metar'].fetchMetarReport(station)
                except IOError :
                    self.stations[station]['lastRefresh'] = 0 # force a refresh next time
                    self.stations[station]['metar'].valid = 0
                else :
                    self.stations[station]['lastRefresh'] = time.time()
        else:
            skeys = self.stations.keys()
            for key in skeys:
                if key not in self.station:
                    del self.stations[key]
            for key in self.station:
                if key not in skeys:
                    try :
                        self.stations[key] = {}
                        self.stations[key]['metar'] = MetarReport(key)
                    except IOError :
                        self.stations[key]['lastRefresh'] = 0 # force a refresh next time
                        self.stations[key]['metar'].valid = 0
                    else :
                        self.stations[key]['lastRefresh'] = time.time()

    security.declareProtected("Manage ZWeatherApplets", "editStation")
    def editStation(self, station, minute, REQUEST=None) :
        """Edits the station code."""
        if REQUEST is not None:
            self.station = station[0].splitlines()
        self.minute  = minute
        self.refresh()
        if REQUEST is not None :
            REQUEST.RESPONSE.redirect("editForm")

    security.declareProtected("View", "getWeather")
    security.declareProtected("View", "getFullReport")
    security.declareProtected("View", "getTemperatureCelsius")
    security.declareProtected("View", "getTemperatureFahrenheit")
    security.declareProtected("View", "getDewPointCelsius")
    security.declareProtected("View", "getDewPointFahrenheit")
    security.declareProtected("View", "getWindSpeed")
    security.declareProtected("View", "getWindDirection")
    security.declareProtected("View", "getWindCompass")
    security.declareProtected("View", "getVisibilityKilometers")
    security.declareProtected("View", "getVisibilityMiles")
    security.declareProtected("View", "getHumidity")
    security.declareProtected("View", "getPressure")
    security.declareProtected("View", "getRawMetarCode")
    security.declareProtected("View", "getSkyConditions")
    security.declareProtected("View", "getStationName")
    security.declareProtected("View", "getStationCity")
    security.declareProtected("View", "getStationCountry")
    security.declareProtected("View", "getCycle")
    security.declareProtected("View", "getStationPosition")
    security.declareProtected("View", "getStationLatitude")
    security.declareProtected("View", "getStationLongitude")
    security.declareProtected("View", "getStationLatitudeFloat")
    security.declareProtected("View", "getStationLongitudeFloat")
    security.declareProtected("View", "getStationAltitude")
    security.declareProtected("View", "getTime")
    security.declareProtected("View", "getISOTime")
    security.declareProtected("View", "getPixmap")

    security.declareProtected("View", "getPixmapURL")
    def getPixmapURL(self, station) :
        """Returns the suggested pixmap url."""
        pix = self.stations[station]['metar'].getPixmap()
        if pix is not None :
            return "%s.png" % pix

    def _setPropValue(self, id, value) :
        """Wrapper to get a new report every time the station property is changed."""
        PropertyManager._setPropValue(self, id, value)
        if id == "station" :
            self.refresh(force=1)

InitializeClass( WeatherTool )
