# Zope Weather Applet - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ZWeatherApplet.py,v 1.6 2002/12/08 21:28:19 jalet Exp $
#
# $Log: ZWeatherApplet.py,v $
# Revision 1.6  2002/12/08 21:28:19  jalet
# getStationHeight ==> getStationAltitude
#
# Revision 1.5  2002/12/02 10:22:46  jalet
# Version number changed to 1.50
# Several modifications, see NEWS file
#
# Revision 1.4  2002/08/01 22:15:55  jalet
# IOError catched now when there's a network problem.
#
# Revision 1.3  2002/07/29 16:28:41  jalet
# Bad leading <br />
#
# Revision 1.2  2002/07/29 10:24:42  jalet
# Moved some stuff to make the help work correctly.
#
# Revision 1.1.1.1  2002/07/29 07:34:02  jalet
# Initial import into CVS
#
#

# This software contains code adapted from the Gnome Weather applet
# written in C language by Spiros Papadimitriou and licensed
# under the GNU General Public License of the Free Software Foundation

# This software contains pictures from the Evolution Mail User Agent
# written by Ximian and distributed under the GNU General Public
# License of the Free Software Foundation

import time

from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import Implicit
from Globals import DTMLFile, Persistent, InitializeClass, MessageDialog
from AccessControl.Role import RoleManager
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem, Item_w__name__

from pymetar import MetarReport

from Products.ZWeatherApplet.help import IZWeatherApplet

manage_addZWeatherAppletForm = DTMLFile('dtml/add', globals())

def manage_addZWeatherApplet(self, id, title='', station=None, REQUEST=None) :
    """Adds a ZWeatherApplet instance in the current container object."""
    if station is not None :
        p = ZWeatherApplet(id, title, station)
        dest = self.Destination()
        dest._setObject(id, p)
        if REQUEST :
            REQUEST.RESPONSE.redirect("manage_main")

class ZWeatherApplet(Implicit, Persistent, SimpleItem, PropertyManager, Item_w__name__, MetarReport) :
    """ZWeatherApplet implementation"""

    __implements__ = IZWeatherApplet.IZWeatherApplet

    _properties = (
                      {'id':'title', 'type': 'string', 'mode':'w'},
                      {'id':'station', 'type': 'string', 'mode':'w'},
                      {'id':'minute', 'type': 'int', 'mode':'w'},
                  )

    meta_type = "ZWeatherApplet"

    security = ClassSecurityInfo()
    security.declareObjectProtected("View")

    def __init__(self, id, title = '', station = 'RCSS') :
        """Initialization."""
        self.id = id
        self.title = title
        self.station = station
        self.minute = 15
        self.stations = {}
        self.lastRefresh = {}
        self.lastRefresh[station] = 0
        self.editStation(station)

    manage_options = (
          { "label" : "Edit", "action" : "editForm" },
          { "label" : "View", "action" : "index_html"},
        ) + PropertyManager.manage_options + SimpleItem.manage_options

    security.declareProtected("View management screens", "editForm")
    editForm = DTMLFile("dtml/change", globals())

    security.declareProtected("View", "index_html")
    def index_html(self, station='', REQUEST={}, RESPONSE=None, **kw) :
        """Displays the ZWeatherApplet information."""
        if station:
            return self.Weather(station)
        else:
            return self.Weather(self.station)

    security.declareProtected("View", "__call__")
    def __call__(self, station='', REQUEST={}, RESPONSE=None, **kw) :
        """Displays the ZWeatherApplet information."""
        if station:
            return self.Weather(station)
        else:
            return self.Weather(self.station)

    security.declareProtected("View", "Weather")
    def Weather(self, station='self.station') :
        """Converts the weather information to HTML."""
        self.refresh(station=station)
        metar = self.stations[station]['metar']
        if metar.valid :
            ihtml = ""
            pix = self.getPixmapURL(station)
            if pix is not None :
                ihtml += '<img src="%s" alt="%s" title="%s" style="width:16px;height:16px" />&nbsp;' % (pix, metar.getPixmap(), metar.getPixmap())
            sname = self.Localizer.Plone(metar.getStationName())
            if sname is not None :
                ihtml += '<span class="station">%s</span>' % sname
            ihtml += "<br />"
            t = metar.getTemperatureCelsius()
            if t is not None :
                ihtml += '<span class="temperature">%s: %.1f&deg;C</span>&nbsp;&nbsp;<br />' % (self.Localizer.Plone('Temperature'), t)
            h = metar.getHumidity()
            if h is not None :
                ihtml += '<span class="humidity">%s: %.0f%%</span>&nbsp;&nbsp;<br />' % (self.Localizer.Plone('Humidity'), h)
            p = metar.getPressure()
            if p is not None :
                ihtml += '<span class="pressure">%s: %.0f hPa</span><br />' % (self.Localizer.Plone('Pressure'), p)
            c = metar.getWindCompass()
            if c is None :
                c = ""
            else :
                c = "%s wind" % c
            s = metar.getWindSpeed()
            if s is not None :
                ihtml += '<span class="wind">%s: %.1f m/s %s</span><br />' % (self.Localizer.Plone('Wind'), s, self.Localizer.Plone(c))
            s = metar.getWeather()
            if s is not None :
                ihtml += '<span class="weather">%s</span><br />' % self.Localizer.Plone(s)
            if ihtml.startswith("<br />") :
                ihtml = ihtml[6:]
            if ihtml.endswith("<br />") :
                ihtml = ihtml[:-6]
            return ihtml


    security.declareProtected("View", "refresh")
    def refresh(self, force=0, station='') :
        """Refreshes the weather information from NOAA's Metar server, if needed.

           force : set this to 1 if you want to fetch a new report even if the
                   cache hasn't expired yet.
        """
        if station and not self.lastRefresh.has_key(station):
            self.lastRefresh[station] = 0
        if station and not self.stations.has_key(station):
            self.editStation(station)
        if station and force or station and ((time.time() - self.lastRefresh[station]) > self.CACHEDURATION()) :
            try :
                self.stations[station]['metar'].fetchMetarReport(station)
            except IOError :
                self.lastRefresh[station] = 0 # force a refresh next time
                self.stations[station]['metar'].valid = 0
            else :
                self.lastRefresh[station] = time.time()


    security.declareProtected("Manage ZWeatherApplets", "CACHEDURATION")
    def CACHEDURATION(self):
        return self.minute * 60

    security.declareProtected("Manage ZWeatherApplets", "editStation")
    def editStation(self, station, minute, REQUEST=None) :
        """Edits the station code."""
        self.station = station
        self.minute = minute
        if station and not self.stations.has_key(station):
            self.stations[station] = {}
            self.stations[station]['metar'] = MetarReport(station)
        if REQUEST is not None :
            REQUEST.RESPONSE.redirect("editForm")

    security.declareProtected("View", "getWeather")
    security.declareProtected("View", "getFullReport")
    security.declareProtected("View", "getTemperatureCelsius")
    security.declareProtected("View", "getTemperatureFahrenheit")
    security.declareProtected("View", "getDewPointCelsius")
    security.declareProtected("View", "getDewPointFahrenheit")
    security.declareProtected("View", "getWindSpeed")
    security.declareProtected("View", "getWindDirection")
    security.declareProtected("View", "getWindCompass")
    security.declareProtected("View", "getVisibilityKilometers")
    security.declareProtected("View", "getVisibilityMiles")
    security.declareProtected("View", "getHumidity")
    security.declareProtected("View", "getPressure")
    security.declareProtected("View", "getRawMetarCode")
    security.declareProtected("View", "getSkyConditions")
    security.declareProtected("View", "getStationName")
    security.declareProtected("View", "getStationCity")
    security.declareProtected("View", "getStationCountry")
    security.declareProtected("View", "getCycle")
    security.declareProtected("View", "getStationPosition")
    security.declareProtected("View", "getStationLatitude")
    security.declareProtected("View", "getStationLongitude")
    security.declareProtected("View", "getStationLatitudeFloat")
    security.declareProtected("View", "getStationLongitudeFloat")
    security.declareProtected("View", "getStationAltitude")
    security.declareProtected("View", "getTime")
    security.declareProtected("View", "getISOTime")
    security.declareProtected("View", "getPixmap")

    security.declareProtected("View", "getPixmapURL")
    def getPixmapURL(self, station) :
        """Returns the suggested pixmap url."""
        pix = self.stations[station]['metar'].getPixmap()
        if pix is not None :
            return "%s/misc_/%s/%s" % (self.absolute_url(), self.__class__.__name__, pix)

    def _setPropValue(self, id, value) :
        """Wrapper to get a new report every time the station property is changed."""
        PropertyManager._setPropValue(self, id, value)
        if id == "station" :
            self.refresh(force=1)

InitializeClass(ZWeatherApplet)
